package lsa;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Date;
import java.util.Iterator;
import java.io.PrintStream;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Hit;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Searcher;

public class SearchFiles {
	private static final String FIELD_PATH = "path";
	private static double THRESHOLD = 0.2f;
	static File INDEX_DIR = new File("index");

	  /** Index all text files under a directory. */
	  public static void indexing(String[] args) {
	    INDEX_DIR = new File(args[0]);
	    if (INDEX_DIR.exists()) {
	      System.out.println("Index exists already: " +INDEX_DIR);
	      return;
	    }    
	    final File docDir = new File(args[1]);
	    if (!docDir.exists() || !docDir.canRead()) {
	      System.out.println("Document directory '" +docDir.getAbsolutePath()+ "' does not exist or is not readable, please check the path");
	      return;
	    }	    
	    Date start = new Date();
	    try {
	      // StandardAnalyzer analyzer = new StandardAnalyzer();
	      Analyzer analyzer = new PorterStemAnalyzer();
	      IndexWriter writer = new IndexWriter(INDEX_DIR, analyzer, true);
	      System.out.println("Indexing to directory '" +INDEX_DIR+ "'...");
	      indexDocs(writer, docDir);
	      System.out.println("Optimizing...");
	      writer.optimize();
	      writer.close();
	      Date end = new Date();
	      System.out.println(end.getTime() - start.getTime() + " total milliseconds");
	    } catch (IOException e) {
	      System.out.println(" caught a " + e.getClass() +
	       "\n with message: " + e.getMessage());
	    }
	  }
	  static void indexDocs(IndexWriter writer, File file)
	    throws IOException {
	    if (file.canRead()) {
	      if (file.isDirectory()) {
	        String[] files = file.list();
	        if (files != null) {
	          for (int i = 0; i < files.length; i++) {
	            indexDocs(writer, new File(file, files[i]));
	          }
	        }
	      } else {
	        System.out.println("adding " + file);
	        try {
	          writer.addDocument(FileDocument.Document(file));
	        }
	        catch (FileNotFoundException fnfe) {
	          ;
	        }
	      }
	    }
	  }
	/** Simple command-line based search demo. */
	public static void main(String[] args) throws Exception {
		indexing(args);
		String index = args[0];
		String field = "contents";
		IndexReader reader = IndexReader.open(index);
		Searcher searcher = new IndexSearcher(reader);
		// Analyzer analyzer = new StandardAnalyzer();
		Analyzer analyzer = new PorterStemAnalyzer();
		QueryParser parser = new QueryParser(field, analyzer);
		Query query = parser.parse(args[2]);
		Hits hits = searcher.search(query);
		System.out.println("Number of hits: " + hits.length());
		Iterator it = hits.iterator();
		while (it.hasNext()) {
			Hit hit = (Hit) it.next();
			Document document = hit.getDocument();
			String path = document.get(FIELD_PATH);
			System.out.println("Hit: " + path);
		}		
//		int n = reader.numDocs();
//		System.out.println("n docs = " + n);
//		double sum1[] = new double[n];
//		double sum2[][] = new double[n][n];
//		for (int i=0; i<n; i++) {
//			sum1[i] = 0f;
//			for (int j=0; j<n; j++)
//				sum2[i][j] = 0f;
//		}
//		int nterms = 0;
//		for (TermEnum terms = reader.terms(); terms.next();) {
//			Term t = terms.term();
//			System.out.println("terms = " + t.text());
//			nterms ++;
//			if (t.field().equalsIgnoreCase(field)) {
//				Query query = parser.parse(t.text());
//				Hits hits = searcher.search(query);
//				int m = hits.length();
//				for (int i=0; i< m; i++) {
//					double f = hits.score(i);
//					int I = hits.id(i);
//					sum1[I] += f*f;
//				}
//				for (int i=0; i< m-1; i++) {
//					for (int j=i+1; j< m; j++) {
//						int I = hits.id(i);
//						int J = hits.id(j);
//						if (I > J) {
//							int tmp = I;
//							J = I;
//							I = tmp;
//						}
//						sum2[I][J] += hits.score(i) * hits.score(j); 
//					}
//				}
//			}
//		}
//		System.out.println("n terms = " + nterms);
//		for (int i=0; i<n-1; i++) {
//			for (int j=i+1; j<n; j++) {
//				double cosine = sum2[i][j] / Math.sqrt(sum1[i])/ Math.sqrt(sum1[j]);
//				if (cosine> THRESHOLD && classify_document(reader,i) != classify_document(reader,j)) {
//					System.out.println("cosine: " + cosine);
//					print_document(reader, i);
//					print_document(reader, j);
//				}
//			}
//		}
		reader.close();
	}

	private static int classify_document(IndexReader reader, int i)
		throws IOException {
		Document doc = reader.document(i);
		String path = doc.get("path");
		  if (path != null && path.indexOf("high")>0) {
			  return 1;
		  } 
		  if (path != null && path.indexOf("low")>0) {
			  return 0;
		  } 
		  return -1;
	}

	
	private static void print_document(IndexReader reader, int i)
			throws IOException {
		Document doc = reader.document(i);
		String path = doc.get("path");
		  if (path != null) {
		    System.out.println(i + ". " + path);
		    String title = doc.get("title");
		    if (title != null) {
		      System.out.println("   Title: " + doc.get("title"));
		    }
		  } else {
		    System.out.println((i+1) + ". " + "No path for this document");
		  }
	}
}
