package lsa;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.io.PrintStream;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Searcher;

public class TermDoc {
	// private static double THRESHOLD = 0.2f;
	static File INDEX_DIR = new File("index");

	  /** Index all text files under a directory. */
	  public static void indexing(String[] args) {
	    INDEX_DIR = new File(args[0]);
	    if (INDEX_DIR.exists()) {
	      System.out.println("Index exists already: " +INDEX_DIR);
	      return;
	    }    
	    final File docDir = new File(args[1]);
	    if (!docDir.exists() || !docDir.canRead()) {
	      System.out.println("Document directory '" +docDir.getAbsolutePath()+ "' does not exist or is not readable, please check the path");
	      return;
	    }	    
	    Date start = new Date();
	    try {
	      StandardAnalyzer analyzer = new StandardAnalyzer();
		  IndexWriter writer = new IndexWriter(INDEX_DIR, analyzer, true);
	      System.out.println("Indexing to directory '" +INDEX_DIR+ "'...");
	      indexDocs(writer, docDir);
	      System.out.println("Optimizing...");
	      writer.optimize();
	      writer.close();
	      Date end = new Date();
	      System.out.println(end.getTime() - start.getTime() + " total milliseconds");
	    } catch (IOException e) {
	      System.out.println(" caught a " + e.getClass() +
	       "\n with message: " + e.getMessage());
	    }
	  }
	  static void indexDocs(IndexWriter writer, File file)
	    throws IOException {
	    if (file.canRead()) {
	      if (file.isDirectory()) {
	        String[] files = file.list();
	        if (files != null) {
	          for (int i = 0; i < files.length; i++) {
	            indexDocs(writer, new File(file, files[i]));
	          }
	        }
	      } else {
	        System.out.println("adding " + file);
	        try {
	          writer.addDocument(FileDocument.Document(file));
	        }
	        catch (FileNotFoundException fnfe) {
	          ;
	        }
	      }
	    }
	  }
        static String docFolder = "";
	/** Simple command-line based search demo. */
	public static void main(String[] args) throws Exception {
		indexing(args);
		docFolder = args[1];
		String index = args[0];
		String field = "contents";
		// THRESHOLD = Float.parseFloat(args[2]);
		IndexReader reader = IndexReader.open(index);
		Searcher searcher = new IndexSearcher(reader);
		Analyzer analyzer = new StandardAnalyzer();
		QueryParser parser = new QueryParser(field, analyzer);
		int n = reader.numDocs();
		System.out.println("n docs = " + n);
		int nterms = 0;
		HashMap termdocs = new HashMap();
                for (int i=0; i<n; i++) {
		   String p = classify_document(reader, i);
		   PrintWriter pw = (PrintWriter) termdocs.get(p);
		   if (pw == null) {
		      pw = new PrintWriter(new File("termdoc."+ p));
		      termdocs.put(p, pw);
		   }
		   pw.print(name_document(reader, i) + " ");
                }
		/*
		 for (TermEnum terms = reader.terms(); terms.next();) {
			Term t = terms.term();
			System.out.println(t.text() + " ");
		 }*/
		 for (Iterator it = termdocs.keySet().iterator(); it.hasNext(); ) {
			 String p = (String) it.next();
			 PrintWriter pw = (PrintWriter) termdocs.get(p);
			 pw.println();
			 for (TermEnum terms = reader.terms(); terms.next();) {
				Term t = terms.term();
				if (t.field().equalsIgnoreCase(field)) {
					nterms ++;
					pw.print("\"" + t.text() + "\" ");
					Query query = parser.parse(t.text());
					Hits hits = searcher.search(query);
					int m = hits.length();
					for (int I=0; I<n; I++) {
					    boolean hit = false;
					    double f = 0;
					    if (classify_document(reader, I).equals(p)) {
						    for (int i=0; i< m; i++) {
							int J = hits.id(i);
							if (I==J) {
								f = hits.score(i);
								hit = true;
								break;
							}
						    }
						    if (hit) {
							pw.print(f + " ");
						    } else {
							pw.print(0 + " ");
						    }
					    }
					}
					pw.println();
				}
			}
			pw.close();
		 }
		reader.close();
	}

	private static String name_document(IndexReader reader, int i)
		throws IOException {
		Document doc = reader.document(i);
		String path = doc.get("path");
		return path.substring(path.lastIndexOf("/")+1);
	}

	private static String classify_document(IndexReader reader, int i)
		throws IOException {
		Document doc = reader.document(i);
		String path = doc.get("path");
		String p = path;
		if (path != null) {
			p = path.substring(docFolder.length() + 1);
			if (p.indexOf("/")>0) {
				p = p.substring(0, p.indexOf("/"));
			} else if (docFolder.lastIndexOf("/")>0) {
				p = docFolder.substring(docFolder.lastIndexOf("/")+1);
			} else {
				p = docFolder;
			}
		} 
		return p;
	}
	
	private static void print_document(IndexReader reader, int i)
			throws IOException {
		Document doc = reader.document(i);
		String path = doc.get("path");
		  if (path != null) {
		    System.out.println(i + ". " + path);
		    String title = doc.get("title");
		    if (title != null) {
		      System.out.println("   Title: " + doc.get("title"));
		    }
		  } else {
		    System.out.println((i+1) + ". " + "No path for this document");
		  }
	}
}
